// filename  FuzzyLogic.C
// simple fuzzy logic motor controller
// tachometer sensor attached to PortAD channel 2
// actuator 8 bit output from Port T 
 
//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing", 
//     Thomsen, copyright (c) 2006,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002,
//    Jonathan W. Valvano 3/17/07

// You may use, edit, run or distribute this file 
//    as long as the above copyright notices remain

#include <hidef.h>      /* common defines and macros */
#include <mc9s12c32.h>     /* derivative information */
#include "PLL.H"
#include "ADC.H"

#pragma LINK_INFO DERIVATIVE "mc9s12c32"

short SpeedError;    // 0 to 255, 128 means no error
short Acceleration;  // 0 to 255, 128 means no acceleration
short U;             // 0 to 255, 0 means no power
short dU;            // -128 to +127, 0 means no power change
short Xstar;         // desired speed, 0 to 255
short Xlast;         // previous speed, 0 to 255
short Xprime;        // estimated speed, 0 to 255
short Bounds(short);
short FuzzyLogic(short SpeedError,short Acceleration);

void main(void) {
  PLL_Init();     // running at 24MHz
  ADC_Init();     // Activate ADC 
  DDRT = 0xFF;    // PortT are output pins
  U = 0;          // U is actuator control
  Xstar = 512;    // desired speed
  Xlast = 0;      // previous speed
  EnableInterrupts;
  for(;;) {
    Xprime = ADC_In(0x82);  // estimated speed
    SpeedError = Xprime-Xstar+128;
    SpeedError = Bounds(SpeedError); // scale to 0 to 255 
    Acceleration = Xprime-Xlast+128;
    Acceleration = Bounds(Acceleration);
    Xlast = Xprime;     // setup for next
    dU = FuzzyLogic(SpeedError,Acceleration);
    U = Bounds(U+dU);
    PTT = (unsigned char)U;    // set DAC actuator 
  }

}

//*******Bounds***********
// Input signed 16-bit
// check for bounds so output is 0 to 255
short Bounds(short data){ 
  if(data>255){ 
    data = 255; // make sure it is within bounds
  }
  if(data<0){   
    data = 0;   // 0 <= data <= +255
  }
  return data;
}

