; ********* Fuzzy.asm ******************
; DC motor, Fuzzy Logic control
; Copyright 2006 by Jonathan W. Valvano, valvano@mail.utexas.edu 
;    You may use, edit, run or distribute this file 
;    as long as the above copyright notice remains 
; Last modified 11/21/05 by Jonathan W. Valvano
 
;  This example accompanies the books
;   "Embedded Microcomputer Systems: Real Time Interfacing", Brooks-Cole, copyright (c) 2006,
;   "Introduction to Embedded Microcomputer Systems: 
;    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002
RAM:     section  

; Fuzzy Membership sets
; input membership variables
          absentry fuzvar 
fuzvar:   ds.b 8    ; inputs
veryfast: equ  0    ; speed very fast
fast:     equ  1    ; speed too fast
ok:       equ  2    ; speed OK
slow:     equ  3    ; speed too slow
veryslow: equ  4    ; speed very slow
up:       equ  5    ; speed increasing
same:     equ  6    ; speed constant
down:     equ  7    ; speed decreasing

;output membership variables
          absentry fuzout 
fuzout:   ds.b 5    ; outputs   
bigadd:   equ  8    ; add a lot of power to system
add:      equ  9    ; add some power to system
zero:     equ  10   ; leave power as is
sub:      equ  11   ; subtract some power from system
bigsub:   equ  12   ; subtract a lot power from system

EEPROM:   section  
; fuzzification
s_tab:    dc.b  $B0,$FF,$08,$00   ; very fast
          dc.b  $80,$D0,$08,$08   ; fast
          dc.b  $60,$A0,$08,$08   ; OK
          dc.b  $30,$80,$08,$08   ; slow
          dc.b  $00,$50,$00,$08   ; very slow
a_tab:    dc.b  $80,$FF,$08,$00   ; increasing speed
          dc.b  $60,$A0,$08,$08   ; constant speed
          dc.b  $00,$80,$00,$08   ; decreasing speed
rules:
;          dc.b  a,b,c,d,$FE,FV,$FE
          dc.b  veryfast,up,$FE,bigsub,$FE
          dc.b  veryfast,same,$FE,bigsub,$FE
          dc.b  veryfast,down,$FE,sub,$FE
          dc.b  fast,up,$FE,sub,$FE  
          dc.b  fast,same,$FE,sub,$FE
          dc.b  fast,down,$FE,zero,$FE  
          dc.b  ok,up,$FE,sub,$FE  
          dc.b  ok,same,$FE,zero,$FE  
          dc.b  ok,down,$FE,add,$FE  
          dc.b  slow,up,$FE,zero,$FE  
          dc.b  slow,same,$FE,add,$FE  
          dc.b  slow,down,$FE,add,$FE  
          dc.b  veryslow,up,$FE,add,$FE
          dc.b  veryslow,same,$FE,bigadd,$FE  
          dc.b  veryslow,down,$FE,bigadd,$FE
          dc.b  $FF  
addsingleton: 
     dc.b 128+75,128+3,128,128-3,128-75  ; 128 subtracted

          absentry FuzzyLogic 
; Stack Parameter 6,sp  =SpeedError, 0 to 255, 128 means no error
; RegB  is 8-bit Acceleration, 128 means no acceleration
; return dU, -128 to +127, 0 means no change    
FuzzyLogic:  
    pshx
    pshb           ; save Acceleration     
    ldaa 6,sp      ; RegA=SpeedError, 0 to 255
    ldx  #s_tab
    ldy  #fuzvar
    mem
    mem
    mem
    mem
    mem

    pula    ;RegA=Acceleration, 0 to 255
    ldx  #a_tab
    mem
    mem
    mem

    ldab #5     ; number of output fuzzy membership sets
cloop:
    clr  1,y+   ; clear output fuzzy variables
    dbne b,cloop
    ldx  #rules
    ldy  #fuzvar
    ldaa #$FF
    rev
    ldy  #fuzout
    ldx  #addsingleton
    ldab #5
    wav
    ediv         ; Y has weighted average, 0 to 255
    leay -128,y  ; adjust for offset, -128 to +127
    tfr  y,d     ; return dpower
    pulx
    rts